Index: src/builtins/x64/builtins-x64.cc |
diff --git a/src/builtins/x64/builtins-x64.cc b/src/builtins/x64/builtins-x64.cc |
index c71179a947c7a3e460caf433bf3ac76c89f5d340..a292d70c71c6a4cd4878b9321ba12b078427e311 100644 |
--- a/src/builtins/x64/builtins-x64.cc |
+++ b/src/builtins/x64/builtins-x64.cc |
@@ -1002,6 +1002,7 @@ |
__ cmpl(index, Immediate(2)); |
__ j(less, &gotta_call_runtime); |
+ // Find literals. |
// r14 : native context |
// r9 : length / index |
// r8 : optimized code map |
@@ -1018,6 +1019,17 @@ |
__ movp(temp, FieldOperand(temp, WeakCell::kValueOffset)); |
__ cmpp(temp, native_context); |
__ j(not_equal, &loop_bottom); |
+ // Literals available? |
+ __ movp(temp, FieldOperand(map, index, times_pointer_size, |
+ SharedFunctionInfo::kOffsetToPreviousLiterals)); |
+ __ movp(temp, FieldOperand(temp, WeakCell::kValueOffset)); |
+ __ JumpIfSmi(temp, &gotta_call_runtime); |
+ |
+ // Save the literals in the closure. |
+ __ movp(FieldOperand(closure, JSFunction::kLiteralsOffset), temp); |
+ __ movp(r15, index); |
+ __ RecordWriteField(closure, JSFunction::kLiteralsOffset, temp, r15, |
+ kDontSaveFPRegs, EMIT_REMEMBERED_SET, OMIT_SMI_CHECK); |
// Code available? |
Register entry = rcx; |
@@ -1026,7 +1038,7 @@ |
__ movp(entry, FieldOperand(entry, WeakCell::kValueOffset)); |
__ JumpIfSmi(entry, &try_shared); |
// Found code. Get it into the closure and return. |
__ leap(entry, FieldOperand(entry, Code::kHeaderSize)); |
__ movp(FieldOperand(closure, JSFunction::kCodeEntryOffset), entry); |
__ RecordWriteCodeEntryField(closure, entry, r15); |
@@ -1057,7 +1069,7 @@ |
__ cmpl(index, Immediate(1)); |
__ j(greater, &loop_top); |
- // We found no code. |
+ // We found neither literals nor code. |
__ jmp(&gotta_call_runtime); |
__ bind(&try_shared); |
@@ -1026,7 +1038,7 @@ |
__ movp(entry, FieldOperand(entry, WeakCell::kValueOffset)); |
__ JumpIfSmi(entry, &try_shared); |
- // Found code. Get it into the closure and return. |
+ // Found literals and code. Get them into the closure and return. |
__ leap(entry, FieldOperand(entry, Code::kHeaderSize)); |
__ movp(FieldOperand(closure, JSFunction::kCodeEntryOffset), entry); |